<?php
session_start();
/**
 * Create/edit club.
 * This page validates and saves, as well as showing the form.
 * Course id passed in GET if it's an edit.
 * POST data present for fields if there is data to be validated. That
 * happens for both edit and insert.
 */
require_once 'library/useful-functions.php';
$errorMessage = '';
// Load default values for club fields.
// Field variables
// They'll be replaced for an existing club.
$clubName = '';
$clubCategory = null;
$clubSponsored = null;
$clubComments = null;
// Was a club id given?
$clubId = getParamFromGet('club_id');
if (! is_null($clubId)) {
    // Id given. It's an edit.
    $errorMessage = checkClubId($clubId);
    if ($errorMessage == '') {
        $clubEntity = getClubWithId($clubId);
        if (is_null($clubEntity)) {
            $errorMessage = "Sorry, something went wrong loading club $clubId.";
        }
        else {
            // Set vars for club fields.
            $clubName = $clubEntity['name'];
            if (! is_null($clubEntity['category'])) {
                $clubCategory = $clubEntity['category'];
            }
            if (! is_null($clubEntity['sponsored'])) {
                $clubSponsored = $clubEntity['sponsored'];
            }
            if (! is_null($clubEntity['comments'])) {
                $clubComments = $clubEntity['comments'];
            }
        }
    }
}
// Is there post data?
if ($errorMessage == '' && $_POST) {
    // User filled in data and sent it to this page.
    // Grab the data sent.
    $clubName = getParamFromPost('name');
    $clubCategory = getParamFromPost('category');
    $clubSponsored = getParamFromPost('sponsored');
    // If sponsored is missing, then the club is not sponsored.
    if (is_null($clubSponsored)) {
        $clubSponsored = 'n';
    }
    $clubComments = getParamFromPost('comments');
    // Validate.
    $errorMessage .= checkName($clubName);
    $errorMessage .= checkClubCategory($clubCategory);
    $errorMessage .= checkClubSponsored($clubSponsored);
    // Is everything OK?
    if ($errorMessage == '') {
        // Yes.
        // Save the data.
        // Put it into session variables.
        $_SESSION['club_id'] = $clubId;
        $_SESSION['club_name'] = $clubName;
        $_SESSION['club_category'] = $clubCategory;
        $_SESSION['club_sponsored'] = $clubSponsored;
        $_SESSION['club_comments'] = $clubComments;
        // Off to processing.
        header('Location: club-save.php');
        exit();
    }
}
?><!doctype html>
<html lang="en">
    <head>
        <?php
        require_once 'library/head.php';
        ?>
    </head>
    <body>
        <?php
        // Work out page title. Add for club of 0, Edit for anything else.
        $pageTitle = getPageTitle($clubId);
        require_once 'library/top.php';
        print "<h1>$pageTitle</h1>\n";
        // If there's an error message, just show that.
        if ($errorMessage != '') {
            print "<p class='error-message'>$errorMessage</p>";
        }
//        else {
            // No error message.
            if ($clubId != 0) {
                // Show the task bar.
                ?>
                <p class='task-bar'>
                <span class='task-link'><a
                        href='club-view.php?club_id=<?= $clubId ?>'>View</a></span>
                <span class='task-link'><a
                        href='club-confirm-delete.php?club_id=<?= $clubId ?>'>Delete</a></span>
                </p>
                <?php
            }
            ?>
            <form name="edit-club" method="post" action="<?= getFormAction($clubId) ?>">
                <p>
                    <label>Name:
                        <input type="text" name="name" value="<?= $clubName ?>">
                    </label>
                </p>
                <p><?= makeCategoryWidget($clubCategory) ?></p>
                <p><?= makeSponsoredWidget($clubSponsored) ?></p>
                <p>
                    <label>Comments:
                        <textarea name="comments"><?= $clubComments ?></textarea>
                    </label>
                </p>
                <p>
                    <button type="submit">Save</button>
                    <?php
                    // Show the cancel link. For an edit, it's the club's view page.
                    // For an add, it's the club list.
                    if ($clubId == 0) {
                        $link = 'clubs.php';
                    }
                    else {
                        $link = "club-view.php?club_id=$clubId";
                    }
                    ?>
                    <span class="task-link"><a href="<?= $link ?>">Cancel</a></span>
                </p>
            </form>
            <?php
//        }
        require_once 'library/footer.php';
        ?>
    </body>
</html><?php

/**
 * Get the page title, based on club id.
 * Id of 0 or null means it's an add, something else means it's an edit.
 * @param mixed $clubId club id
 * @return string Page title
 */
function getPageTitle($clubId) {
    $pageTitle = '';
    if (is_null($clubId) || $clubId == 0) {
        $pageTitle = 'Add club';
    }
    else {
        $pageTitle = 'Edit club';
    }
    return $pageTitle;
}

/**
 * Work out the ACTION of the form.
 * If club id is 0, it's an add, otherwise an edit.
 * @param int $clubId club id.
 * @return string Value for ACTION.
 */
function getFormAction($clubId) {
    $formAction = 'club-add-edit.php';
    if ($clubId > 0) {
        $formAction .= "?club_id=$clubId";
    }
    return $formAction;
}

/**
 * Make HTML for the category widget.
 * @param mixed $clubCategory Current category.
 * @return string The HTML.
 */
function makeCategoryWidget($clubCategory) {
    $result = "
        Category<br>
            <label>
                <input type='radio' name='category' value='c' ";
    if ($clubCategory == 'c') {
        $result .= 'checked';
    }
    $result .= "
                >
            Core</label><br>

            <label>
                <input type='radio' name='category' value='l' ";
    if ($clubCategory == 'l') {
        $result .= 'checked';
    }
    $result .= "
                >
            Lifestyle</label><br>

            <label>
                <input type='radio' name='category' value='a' ";
    if ($clubCategory == 'a') {
        $result .= 'checked';
    }
    $result .= "
                >
                Academic</label>
    ";
    return $result;
}


/**
 * Make HTML for the sponsored widget.
 * @param mixed $clubSponsored Current y/n value.
 * @return string The HTML.
 */
function makeSponsoredWidget($clubSponsored) {
    $result = "
        <label>
            <input type='checkbox' name='sponsored' ";
    if ($clubSponsored == 'y') {
        $result .= 'checked';
    }
    $result .= "
            >
            Sponsored
        </label>
    ";
    return $result;
}
