<?php
/**
 * Confirm that the user wants to delete a club.
 */
require_once 'library/useful-functions.php';
$errorMessage = '';
// Was the id given?
$clubId = getParamFromGet('club_id');
$errorMessage = checkClubId($clubId);
if ($errorMessage == '') {
    // Does the club have members?
    $memberCount = getCountMembersInClub($clubId);
    if ($memberCount > 0) {
        // Can't delete a club that has members.
        $errorMessage = "Sorry, that club still has members.";
    }
}
?><!doctype html>
<html lang="en">
    <head>
        <?php
        $pageTitle = 'Delete club';
        require_once 'library/head.php';
        ?>
    </head>
    <body>
        <div class="container">
            <?php
            require_once 'library/top.php'
            ?>
            <h1><?= $pageTitle ?></h1>
            <?php
            if ($errorMessage != '') {
                print "<p class='error-message'>$errorMessage</p>";
            }
            else {
                print "<p>Are you sure you want to delete this club?</p>\n";
                /** @var PDO $dbConnection */
                $result = renderClubDeets($clubId);
                if (is_null($result)) {
                    print "<p>Sorry, something went wrong.</p>\n";
                }
                else {
                    print $result;
                    print makeTaskBar($clubId);
                }

            }
            require_once 'library/footer.php';
            ?>
    </body>
</html><?php

/**
 * Make HTML to show the task bar.
 * @param int $clubId club id.
 * @return string HTML.
 */
function makeTaskBar($clubId) {
    $confirmLink = "<a href='club-delete-confirmed.php?club_id=$clubId'>Confirm</a>";
    $cancelLink = "<a href='club-view.php?club_id=$clubId'>Cancel</a>";
    $result = "<p class='task-bar'>
            <span class='task-link'>$confirmLink</span>
            <span class='task-link'>$cancelLink</span>
        </p>\n";
    return $result;
}


