<?php
session_start();
/**
 * Create/edit comedian.
 * This page validates and saves, as well as showing the form.
 * Course id passed in GET if it's an edit.
 * POST data present for fields if there is data to be validated. That
 * happens for both edit and insert.
 */
require_once 'library/useful-functions.php';
$errorMessage = '';
// comedian id to edit is passed in the URL. If not there, must be a create.
// Show an insert by setting the id to 0.
$comedianId = 0;
// Load default values for comedian fields.
// Field variables
// They'll be replaced for an existing comedian.
$comedianName = '';
$comedianComments = null;
// Was a comedian id given?
$comedianId = getParamFromGet('comedian_id');
if (! is_null($comedianId)) {
    // Id found, an edit.
    $errorMessage = checkComedianId($comedianId);
    if ($errorMessage == '') {
        $comedianEntity = getComedianWithId($comedianId);
        if (is_null($comedianEntity)) {
            $errorMessage = "Sorry, something went wrong loading comedian $comedianId.";
        } else {
            // Set vars for comedian fields.
            $comedianName = $comedianEntity['name'];
            $comedianComments = $comedianEntity['comments'];
        }
    }
}
// Is there post data?
if ($errorMessage == '' && $_POST) {
    // User filled in data and sent it to this page.
    // Grab the data sent.
    $comedianName = getParamFromPost('name');
    $comedianComments = getParamFromPost('comments');
    // Validate.
    $errorMessage .= checkName($comedianName);
    // Is everything OK?
    if ($errorMessage == '') {
        // Yes.
        // Save the data.
        // Put it into session variables.
        $_SESSION['comedian_id'] = $comedianId;
        $_SESSION['comedian_name'] = $comedianName;
        $_SESSION['comedian_comments'] = $comedianComments;
        // Off to processing.
        header('Location: comedian-save.php');
        exit();
    }
}
?><!doctype html>
<html lang="en">
    <head>
        <?php
        require_once 'library/head.php';
        ?>
    </head>
    <body>
        <?php
        // Work out page title. Add for comedian of 0, Edit for anything else.
        $pageTitle = getPageTitle($comedianId);
        require_once 'library/top.php';
        print "<h1>$pageTitle</h1>\n";
        // If there's an error message, just show that.
        if ($errorMessage != '') {
            print "<p class='error-message'>$errorMessage</p>";
        }
//        else {
            // No error message.
            if ($comedianId != 0) {
                // Show the task bar.
                ?>
                <p class='task-bar'>
                <span class='task-link'><a
                        href='comedian-view.php?comedian_id=<?= $comedianId ?>'>View</a></span>
                <span class='task-link'><a
                            href='comedian-confirm-delete.php?comedian_id=<?= $comedianId ?>'>Delete</a></span>
                </p>
                <?php
            }
            ?>
            <form name="edit-comedian" method="post" action="<?= getFormAction($comedianId) ?>">
                <p>
                    <label>Name:
                        <input type="text" name="name" value="<?= $comedianName ?>">
                    </label>
                </p>
                <p>
                    <label>Comments:
                        <textarea name="comments"><?= $comedianComments ?></textarea>
                    </label>
                </p>
                <p>
                    <button type="submit">Save</button>
                    <?php
                    // Show the cancel link. For an edit, it's the comedian's view page.
                    // For an add, it's the comedian list.
                    if ($comedianId == 0) {
                        $link = 'comedians.php';
                    }
                    else {
                        $link = "comedian-view.php?comedian_id=$comedianId";
                    }
                    ?>
                    <span class="task-link"><a href="<?= $link ?>">Cancel</a></span>
                </p>
            </form>
            <?php
//        }
        require_once 'library/footer.php';
        ?>
    </body>
</html><?php

/**
 * Get the page title, based on comedian id.
 * Id of 0 or null means it's an add, something else means it's an edit.
 * @param mixed $comedianId comedian id
 * @return string Page title
 */
function getPageTitle($comedianId) {
    $pageTitle = '';
    if (is_null($comedianId) || $comedianId == 0) {
        $pageTitle = 'Add comedian';
    }
    else {
        $pageTitle = 'Edit comedian';
    }
    return $pageTitle;
}

/**
 * Work out the ACTION of the form.
 * If comedian id is 0, it's an add, otherwise an edit.
 * @param int $comedianId comedian id.
 * @return string Value for ACTION.
 */
function getFormAction($comedianId) {
    $formAction = 'comedian-add-edit.php';
    if ($comedianId > 0) {
        $formAction .= "?comedian_id=$comedianId";
    }
    return $formAction;
}

