<?php
/**
 * Confirm that the user wants to delete a comedian.
 */
require_once 'library/useful-functions.php';
$errorMessage = '';
// Was the id given?
if (! isset($_GET['comedian_id'])) {
    $errorMessage = 'Sorry, there is no comedian id.';
}
else {
    $comedianId = sanitizeId($_GET['comedian_id']);
    // Does the comedian have goats who faved them?
    $faveCount = getCountGoatsWithFaveComedian($comedianId);
    if (is_null($faveCount)) {
        $errorMessage = 'Sorry, something went wrong counting goats with face.';
    }
    else {
        if ($faveCount > 0) {
            $errorMessage = "Sorry, that comedian has goats who have marked them as fave.";
        }
    }
}
?><!doctype html>
<html lang="en">
    <head>
        <?php
        $pageTitle = 'Delete comedian';
        require_once 'library/head.php';
        ?>
    </head>
    <body>
        <div class="container">
            <?php
            require_once 'library/top.php'
            ?>
            <h1><?= $pageTitle ?></h1>
            <?php
            if ($errorMessage != '') {
                print "<p class='error-message'>$errorMessage</p>";
            }
            else {
                print "<p>Are you sure you want to delete this comedian?</p>\n";
                /** @var PDO $dbConnection */
                $result = rendercomedianDeets($comedianId);
                if (is_null($result)) {
                    print "<p>Sorry, something went wrong.</p>\n";
                }
                else {
                    print $result;
                    print makeTaskBar($comedianId);
                }

            }
            require_once 'library/footer.php';
            ?>
    </body>
</html><?php

/**
 * Make HTML to show the task bar.
 * @param int $comedianId comedian id.
 * @return string HTML.
 */
function makeTaskBar($comedianId) {
    $confirmLink = "<a href='comedian-delete-confirmed.php?comedian_id=$comedianId'>Confirm</a>";
    $cancelLink = "<a href='comedian-view.php?comedian_id=$comedianId'>Cancel</a>";
    $result = "<p class='task-bar'>
            <span class='task-link'>$confirmLink</span>
            <span class='task-link'>$cancelLink</span>
        </p>\n";
    return $result;
}


