<?php
session_start();
/**
 * Create/edit goat.
 * This page validates and saves, as well as showing the form.
 * Course id passed in GET if it's an edit.
 * POST data present for fields if there is data to be validated. That
 * happens for both edit and insert.
 */
require_once 'library/useful-functions.php';
$errorMessage = '';
// Load default values for goat fields.
// Field variables
// They'll be replaced for an existing goat.
$goatName = '';
$goatGender = null;
$goatFaveComedianId = null;
$goatComments = null;
// Make an array of club memberships, element for every club, true if member.
$goatClubMemberships = [];
// Load comedians.
/** @var PDO $dbConnection */
$comedianRows = getAllComedians();
// Load clubs.
$clubRows = getAllClubs();
// Was a goat id given?
$goatId = getParamFromGet('goat_id');
if (! is_null($goatId)) {
    // Id found, an edit.
    $errorMessage = checkGoatId($goatId);
    if ($errorMessage == '') {
        $goatEntity = getGoatWithId($goatId);
        if (is_null($goatEntity)) {
            $errorMessage = "Sorry, something went wrong loading goat $goatId.";
        }
        else {
            // Set vars for goat fields.
            $goatName = $goatEntity['name'];
            if (! is_null($goatEntity['gender'])) {
                $goatGender = $goatEntity['gender'];
            }
            if (! is_null($goatEntity['fave_comedian'])) {
                $goatFaveComedianId = $goatEntity['fave_comedian'];
            }
            if (! is_null($goatEntity['comments'])) {
                $goatComments = $goatEntity['comments'];
            }
            $goatClubMemberships = getClubsGoatIsIn($goatId);
            if (is_null($goatClubMemberships)) {
                $errorMessage = "Sorry, something went wrong loading club memberships for goat $goatId.";
            }
        }
    }
}

// Is there post data?
if ($errorMessage == '' && $_POST) {
    // User filled in data and sent it to this page.
    // Grab the data sent.
    $goatName = getParamFromPost('name');
    $goatGender = getParamFromPost('gender');
    $goatFaveComedianId = getParamFromPost('fave-comedian');
    if ($goatFaveComedianId == 0) {
        // (Choose) option selected on prior page.
        $goatFaveComedianId = null;
    }
    $goatComments = getParamFromPost('comments');
    $goatClubMemberships = getParamFromPost('club-memberships');
    // Validate.
    $errorMessage .= checkName($goatName);
    // Gender is optional.
    if (! is_null($goatGender)) {
        $errorMessage .= checkGender($goatGender);
    }
    // Fave comedian is optional.
    if (! is_null($goatFaveComedianId)) {
        $errorMessage .= checkComedianId($goatFaveComedianId);
    }
    foreach ($goatClubMemberships as $clubMembership) {
        $errorMessage .= checkClubId($clubMembership);
    }
    // Is everything OK?
    if ($errorMessage == '') {
        // Yes.
        // Save the data.
        // Put it into session variables.
        $_SESSION['goat_id'] = $goatId;
        $_SESSION['goat_name'] = $goatName;
        $_SESSION['goat_gender'] = $goatGender;
        $_SESSION['goat_fave_comedian'] = $goatFaveComedianId;
        $_SESSION['goat_comments'] = $goatComments;
        $_SESSION['goat_clubs'] = $goatClubMemberships;
        // Off to processing.
        header('Location: goat-save.php');
        exit();
    }
}
?><!doctype html>
<html lang="en">
<head>
    <?php
    // Work out page title. Add for goat of 0, Edit for anything else.
    $pageTitle = getPageTitle($goatId);
    require_once 'library/head.php';
    ?>
</head>
<body>
    <?php
    require_once 'library/top.php';
    print "<h1>$pageTitle</h1>\n";
    // If there's an error message, just show that.
    if ($errorMessage != '') {
        print "<p class='error-message'>$errorMessage</p>";
    }
    else {
        // No error message.
        if ($goatId != 0) {
            // Show the edit task bar.
            ?>
            <p class='task-bar'>
                <span class='task-link'><a
                    href='goat-view.php?goat_id=<?= $goatId ?>'>View</a></span>
                <span class='task-link'><a
                            href='goat-confirm-delete.php?goat_id=<?= $goatId ?>'>Delete</a></span>
            </p>
        <?php
        }
        ?>
        <form name="edit-goat" method="post" action="<?= getFormAction($goatId) ?>">
            <p>
                <label>Name:
                    <input type="text" name="name" value="<?= $goatName ?>">
                </label>
            </p>
            <p><?= makeGenderWidget($goatGender) ?></p>
            <p><?= makeFaveComedianWidget($goatFaveComedianId, $comedianRows) ?></p>
            <p><?= makeClubMembershipWidget($goatClubMemberships, $clubRows) ?></p>
            <p>
                <label>Comments:
                    <textarea name="comments"><?= $goatComments ?></textarea>
                </label>
            </p>
            <p>
                <button type="submit">Save</button>
                <?php
                // Show the cancel link. For an edit, it's the goat's view page.
                // For an add, it's the goat list.
                if ($goatId == 0) {
                    $link = 'goats.php';
                }
                else {
                    $link = "goat-view.php?goat_id=$goatId";
                }
                ?>
                <span class="task-link"><a href="<?= $link ?>">Cancel</a></span>
            </p>
        </form>
        <?php
    }
    require_once 'library/footer.php';
    ?>
</body>
</html><?php

/**
 * Get the page title, based on goat id.
 * Id of 0 or null means it's an add, something else means it's an edit.
 * @param mixed $goatId Goat id
 * @return string Page title
 */
function getPageTitle($goatId) {
    $pageTitle = '';
    if (is_null( $goatId) || $goatId == 0) {
        $pageTitle = 'Add goat';
    }
    else {
        $pageTitle = 'Edit goat';
    }
    return $pageTitle;
}

/**
 * Work out the ACTION of the form.
 * If goat id is 0, it's an add, otherwise an edit.
 * @param int $goatId Goat id.
 * @return string Value for ACTION.
 */
function getFormAction($goatId) {
    $formAction = 'goat-add-edit.php';
    if ($goatId > 0) {
        $formAction .= "?goat_id=$goatId";
    }
    return $formAction;
}

/**
 * Make HTML for the gender widget.
 * @param string $goatGender Current gender, f, m, or nothing.
 * @return string The HTML.
 */
function makeGenderWidget($goatGender) {
    $result = "
        Gender<br>
        <input type='radio' id='female' name='gender' value='f' ";
    if ($goatGender == 'f') {
        $result .= 'checked ';
    }
    $result .= ">
        <label for='female'>Female</label><br>
        <input type='radio' id='male' name='gender' value='m' ";
        if ($goatGender == 'm') {
            $result .= 'checked ';
        }
    $result .= ">
        <label for='male'>Male</label>\n";
    return $result;
}


/**
 * Make HTML to render fave comedian widget.
 * @param int $faveComedianId The favorite comedian
 * @param array $comedianRows Available comedians.
 * @return string HTML to make the widget.
 */
function makeFaveComedianWidget($faveComedianId, $comedianRows) {
    $result = "
        <label>Fave comedian<br>
            <select name='fave-comedian'>
                <option value='0'>(Not given)</option>\n";
    foreach ($comedianRows as $comedianRow) {
        $comedianId = $comedianRow['comedian_id'];
        $comedianName = $comedianRow['name'];
        $selected = '';
        if ($comedianId == $faveComedianId) {
            $selected = 'selected';
        }
        $result .= "<option value='$comedianId' $selected>$comedianName</option>\n";
    }
    $result .="
            </select>
        </label>\n";
    return $result;
}

/**
 * Make HTML for club membership widget.
 * @param array $goatClubMemberships The goat's memberships.
 *   'club_id' element in each row has the id of the club the goat is in.
 * @param array $clubRows All of the clubs.
 * @return string HTML to render widget.
 */
function makeClubMembershipWidget(array $goatClubMemberships, array $clubRows) {
    $memberOfClubIds = array_column($goatClubMemberships, 'club_id');
    $result = "
        <label>Club memberships<br>
            <select name='club-memberships[]' multiple size='5'>\n";
    foreach ($clubRows as $clubRow) {
        $clubId = $clubRow['club_id'];
        $clubName = $clubRow['name'];
        $selected = '';
        // Is the goat in the club?
        if (in_array($clubId, $memberOfClubIds)) {
            $selected = 'selected';
        }
        $result .= "<option value='$clubId' $selected>$clubName</option>\n";
    }
    $result .= "
            </select>
        </label>\n";
    return $result;
}
