<?php
session_start();
require_once 'library/useful-functions.php';
// Grab data to save.
$goatId = $_SESSION['goat_id'];
$goatName = $_SESSION['goat_name'];
$goatGender = $_SESSION['goat_gender'];
$goatFaveComedianId = $_SESSION['goat_fave_comedian'];
$goatComments = $_SESSION['goat_comments'];
$goatClubMemberships = $_SESSION['goat_clubs'];
// Check the id. 0 means add, other value means edit.
if ($goatId == 0) {
    global $canChangeData;
    if ($canChangeData) {
        // This is a new goat.
        // Add to the goats table.
        $sql = "
            INSERT INTO goats (
                name, gender, fave_comedian, comments
            )
            VALUES (
                :name, :gender, :comedian, :comments
            )
        ";
        /** @var PDO $dbConnection */
        $stmnt = $dbConnection->prepare($sql);
        $isWorked = $stmnt->execute(
            [
                'name' => $goatName,
                'gender' => $goatGender,
                'comedian' => $goatFaveComedianId,
                'comments' => $goatComments
            ]
        );
        // Get the id of the new goat.
        $goatId = $dbConnection->lastInsertId();
        // Make club membership records, if any.
        if (!is_null($goatClubMemberships)) {
            recordClubMemberships($goatId, $goatClubMemberships);
        }
    }
    else {
        // Can't delete.
        $goatId = 1;
    }
}
else {
    global $canChangeData;
    if ($canChangeData) {
        // It's an edit.
        $sql = "
        UPDATE goats SET
            name = :name,
            gender = :gender,
            fave_comedian = :comedian,
            comments = :comments
        WHERE 
            goat_id = :id;
    ";
        /** @var PDO $dbConnection */
        $stmnt = $dbConnection->prepare($sql);
        $stmnt->execute(
            [
                'id' => $goatId,
                'name' => $goatName,
                'gender' => $goatGender,
                'comedian' => $goatFaveComedianId,
                'comments' => $goatComments,
            ]
        );
        // Erase existing club memberships.
        $sql = "
        DELETE FROM club_members
            WHERE goat_id = :id;
    ";
        $stmnt = $dbConnection->prepare($sql);
        $stmnt->execute(['id' => $goatId]);
        // Make new club membership records.
        if (!is_null($goatClubMemberships)) {
            recordClubMemberships($goatId, $goatClubMemberships);
        }
    }
}
// Redirect to view page.
header("Location: goat-view.php?goat_id=$goatId");
exit();

/**
 * Record club memberships for a goat.
 * @param int $goatId Goat id.
 * @param array $goatClubMemberships Array of club ids.
 */
function recordClubMemberships(int $goatId, array $goatClubMemberships) {
    global $dbConnection;
    foreach ($goatClubMemberships as $clubId) {
        $sql = "
            INSERT INTO club_members (
                goat_id, club_id
            )
            VALUES (
                :goat, :club
            );";
        $stmnt = $dbConnection->prepare($sql);
        $stmnt->execute([
            'goat' => $goatId,
            'club' => $clubId
        ]);
    }
}