<?php
global $canChangeData;
$canChangeData = false;

// Connect to DB.
// Load DB secrets from outside the web root.
require_once 'D:\secret-stuff\goats-db-params.php';

/**
 * Sanitize a value that is supposed to be an id.
 * @param mixed $idToSanitize Value to sanitize.
 * @return mixed Result of sanitation.
 */
function sanitizeId($idToSanitize) {
    $result = filter_var($idToSanitize, FILTER_SANITIZE_NUMBER_INT);
    return $result;
}

/**
 * Get a value for parameter in the GET array.
 * @param string $paramName Name of the parameter.
 * @return string|null Value, or null if not found.
 */
function getParamFromGet(string $paramName) {
    $returnValue = null;
    if (isset($_GET[$paramName])) {
        $returnValue = $_GET[$paramName];
        if ($returnValue == '') {
            $returnValue = null;
        }
    }
    return $returnValue;
}

/**
 * Get a value for parameter in the POST array.
 * @param string $fieldName Name of the field.
 * @return string|null Trimmed value, or null if field not found or is MT.
 */
function getParamFromPost(string $fieldName) {
    $value = null;
    if (isset($_POST[$fieldName])) {
        $value = $_POST[$fieldName];
        if (is_string($value)) {
            $value = trim($value);
        }
        if ($value == '') {
            $value = null;
        }
    }
    return $value;
}

/**
 * @param string $userInput User input to check.
 * @return string Error message, MT if data is OK.
 */
function checkName($userInput) {
    $errorMessage = '';
    if (is_null($userInput)) {
        $errorMessage = "Sorry, you must enter a name.<br>\n";
    }
    return $errorMessage;
}

/**
 * Given a club category code, return the category's name.
 * @param string $categoryCode The code, e.g., c.
 * @return string|null The name, e.g., Core
 */
function getCategoryNameForCode($categoryCode) {
    $categoryName = null;
    if ($categoryCode == 'c') {
        $categoryName = 'Core';
    }
    elseif ($categoryCode == 'l') {
        $categoryName = 'Lifestyle';
    }
    elseif ($categoryCode == 'a') {
        $categoryName = 'Academic';
    }
    return $categoryName;
}

/**
 * Given y or n code, return Yes or No.
 * @param string $code The code, e.g., y.
 * @return string|null The name, e.g., Yes. Null if invalid.
 */
function getYNNameForCode($code) {
    $name = null;
    if ($code == 'y') {
        $name = 'Yes';
    }
    elseif ($code == 'n') {
        $name = 'No';
    }
    return $name;
}

/**
 * Get name of gender, given code.
 * @param string $code The code, e.g., f.
 * @return string|null The name, e.g., Female. Null if invalid.
 */
function getNameForGenderCode($code) {
    $name = null;
    if ($code == 'f') {
        $name = 'Female';
    }
    elseif ($code == 'm') {
        $name = 'Male';
    }
    return $name;
}

/**
 * Check a club category code.
 * @param string $toCheck Value to check.
 * @return string Error message, MT if none
 */
function checkClubCategory($toCheck) {
    $errorMessage = '';
    if (is_null($toCheck)) {
        $errorMessage = "Sorry, category cannot be empty.<br>\n";
    }
    if ($errorMessage == '') {
        if ($toCheck != 'c' && $toCheck != 'l' && $toCheck != 'a') {
            $errorMessage = "Sorry, unknown category: $toCheck.<br>\n";
        }
    }
    return $errorMessage;
}

/**
 * Check a club sponsored code.
 * @param string $toCheck Value to check.
 * @return string Error message, MT if none
 */
function checkClubSponsored($toCheck) {
    $errorMessage = '';
    if (is_null($toCheck)) {
        $errorMessage = "Sorry, sponsored code cannot be empty.<br>\n";
    }
    if ($errorMessage == '') {
        if ($toCheck != 'y' && $toCheck != 'n') {
            $errorMessage = "Sorry, unknown sponsored code: $toCheck.<br>\n";
        }
    }
    return $errorMessage;
}

/**
 * Check a gender code.
 * @param string|null $toCheck Value to check.
 * @return string Error message, MT if none
 */
function checkGender($toCheck) {
    $errorMessage = '';
    if (!is_null($toCheck) && $toCheck != 'f' && $toCheck != 'm') {
        $errorMessage = "Sorry, gender must be female or male.<br>\n";
    }
    return $errorMessage;
}

/**
 * Get comedian record for given id.
 * @param int $comedianId Id to look up.
 * @return array|null Record from DB, or null is not found, or bad id.
 */
function getComedianWithId($comedianId) {
    global $dbConnection;
    $result = null;
    if (! is_null($comedianId) && is_numeric($comedianId)) {
        $sql = "
        select 
            *
        from
            comedians 
        where
            comedians.comedian_id = :comedian_id_to_show;
    ";
        /** @var PDO $dbConnection */
        $stmnt = $dbConnection->prepare($sql);
        $isQueryWorked = $stmnt->execute([':comedian_id_to_show' => $comedianId]);
        if ($isQueryWorked && $stmnt->rowCount() == 1) {
            $result = $stmnt->fetch();
        }
    }
    return $result;
}

/**
 * Get club record for given id.
 * @param int $clubId Id to look up.
 * @return array|null Record from DB, or null is not found, or bad id.
 */
function getClubWithId($clubId) {
    global $dbConnection;
    $result = null;
    if (! is_null($clubId) && is_numeric($clubId)) {
        $sql = "
        select 
            *
        from
            clubs 
        where
            clubs.club_id = :club_id_to_show;
    ";
        /** @var PDO $dbConnection */
        $stmnt = $dbConnection->prepare($sql);
        $isQueryWorked = $stmnt->execute([':club_id_to_show' => $clubId]);
        if ($isQueryWorked && $stmnt->rowCount() == 1) {
            $result = $stmnt->fetch();
        }
    }
    return $result;
}

/**
 * Get goat record for given id.
 * @param int $goatId Id to look up.
 * @return array|null Record from DB, or null is not found, or bad id.
 */
function getGoatWithId($goatId) {
    global $dbConnection;
    $result = null;
    if (! is_null($goatId) && is_numeric($goatId)) {
        $sql = "
        select 
            *
        from
            goats 
        where
            goats.goat_id = :goat_id_to_show;
    ";
        /** @var PDO $dbConnection */
        $stmnt = $dbConnection->prepare($sql);
        $isQueryWorked = $stmnt->execute([':goat_id_to_show' => $goatId]);
        if ($isQueryWorked && $stmnt->rowCount() == 1) {
            $result = $stmnt->fetch();
        }
    }
    return $result;
}

/**
 * Get the clubs a goat is in.
 * @param int $goatId Goat id.
 * @return null|array Records, null if something went wrong.
 */
function getClubsGoatIsIn($goatId) {
    global $dbConnection;
    $result = null;
    if (!is_null($goatId) && is_numeric($goatId)) {
        $sql = "
            select 
                clubs.*
            from
                club_members, clubs 
            where
                club_members.goat_id = :goat_id_to_show
                and club_members.club_id = clubs.club_id
            order by clubs.name;
        ";
        $stmnt = $dbConnection->prepare($sql);
        $isQueryWorked = $stmnt->execute([':goat_id_to_show' => $goatId]);
        if ($isQueryWorked) {
            $result = $stmnt->fetchAll();
        }
    }
    return $result;
}

/**
 * Get all clubs.
 * @return null|array Club rows, null if something broke.
 */
function getAllClubs() {
    global $dbConnection;
    $result = null;
    $sql = "select * from clubs order by name;";
    $stmnt = $dbConnection->prepare($sql);
    $isQueryWorked = $stmnt->execute();
    if ($isQueryWorked) {
        $result = $stmnt->fetchAll();
    }
    return $result;
}

/**
 * Get all comedians.
 * @return null|array Comedian rows, null if something broke.
 */
function getAllComedians() {
    global $dbConnection;
    $result = null;
    $sql = "select * from comedians order by name;";
    $stmnt = $dbConnection->prepare($sql);
    $isQueryWorked = $stmnt->execute();
    if ($isQueryWorked) {
        $result = $stmnt->fetchAll();
    }
    return $result;
}

/**
 * Get all goats.
 * @return null|array Goat rows, null if something broke.
 */
function getAllGoats() {
    global $dbConnection;
    $result = null;
    $sql = "select * from goats order by name;";
    $stmnt = $dbConnection->prepare($sql);
    $isQueryWorked = $stmnt->execute();
    if ($isQueryWorked) {
        $result = $stmnt->fetchAll();
    }
    return $result;
}

/**
 * If passed null, returns MT string, else returns what it was passed.
 * @param $value
 * @return string
 */
function getValueOrMT($value) {
    $result = $value;
    if (is_null($result)) {
        $result = '';
    }
    return $result;
}


/**
 * Make HTML to show list of clubs goat is in.
 * @param array $clubs Clubs goat is in.
 * @return string HTML
 */
function renderClubList($clubs) {
    $result = "<ul>\n";
    foreach ($clubs as $club) {
        $clubId = $club['club_id'];
        $name = $club['name'];
        $link = "<a href='club-view.php?club_id=$clubId'>$name</a>";
        $result .= "<li>$link</li>\n";
    }
    $result .= "</ul>\n";
    return $result;
}

/**
 * Make HTML to render a goat.
 * @param int $goatId The goat's id.
 * @return string|null HTML to show the goat, or null if something broke.
 */
function renderGoatDeets($goatId) {
    global $dbConnection;
    $goatEntity = getGoatWithId($goatId);
    if ( is_null($goatEntity) ) {
        return null;
    }
    $result = "<p>Name: {$goatEntity['name']}</p>\n";
    // Show gender.
    $gender = $goatEntity['gender'];
    if (is_null($gender)) {
        $genderDisplay = "(Not given)";
    }
    else {
        $genderDisplay = getNameForGenderCode($gender);
    }
    $result .= "<p>Gender: $genderDisplay</p>\n";
    // Show fave comedian.
    $faveComedianId = $goatEntity['fave_comedian'];
    if (is_null($faveComedianId)) {
        $faveComedianDisplay = "(Not given)";
    }
    else {
        $comedianEntity = getComedianWithId($faveComedianId);
        $comedianName = $comedianEntity['name'];
        $faveComedianDisplay = "<a href='comedian-view.php?comedian_id=$faveComedianId'>$comedianName</a>";
    }
    $result .= "<p>Fave comedian: $faveComedianDisplay</p>\n";
    $result .= "<p>Comments: {$goatEntity['comments']}</p>\n";
    $result .= "<h3>Clubs</h3>\n";
    // Render clubs the goat is in.
    $clubs = getClubsGoatIsIn($goatId);
    if (count($clubs) == 0) {
        // Goat is not in any clubs.
        $result .= "<p>{$goatEntity['name']} isn't in any clubs.</p>\n";
    }
    else {
        $result .= renderClubList($clubs);
    }
    return $result;
}

/**
 * Make HTML to render a club.
 * @param int $clubId The goat's id.
 * @return string|null HTML to show the club, or null if something broke.
 */
function renderClubDeets($clubId) {
    global $dbConnection;
    $clubEntity = getClubWithId($clubId);
    if ( is_null($clubEntity) ) {
        return null;
    }
    $result = "<p>Name: {$clubEntity['name']}</p>\n";
    $result .= '<p>Category: ' . getCategoryNameForCode($clubEntity['category']) . "</p>\n";
    $result .= '<p>Sponsored: ' . getYNNameForCode($clubEntity['sponsored']) . "</p>\n";
    $result .= "<p>Comments: {$clubEntity['comments']}</p>\n";
    // Show the members.
    $goats = getMembersInClub($clubId);
    if (is_null($goats)) {
        $result .= '<p><strong>Something went wrong looking up members.</strong></p>';
    }
    else {
        if (count($goats) == 0) {
            $result .= '<p><strong>The club has no members.</strong></p>';
        }
        else {
            $result .= "<h3>Members</h3>\n";
            $result .= "<ul>\n";
            foreach ($goats as $goat) {
                $viewUrl = "goat-view.php?goat_id={$goat['goat_id']}";
                $viewLink = "<a href='$viewUrl'>{$goat['name']}</a>";
                $result .= "<li>$viewLink</li>\n";
            }
            $result .= "</ul>\n";
        }
    }
    return $result;
}

/**
 * Get the members in a club.
 * @param int $clubId The club's id.
 * @return array|null Members, or null if something broke.
 */
function getMembersInClub($clubId) {
    global $dbConnection;
    $sql = "SELECT goats.* 
        FROM club_members, goats
        WHERE club_members.club_id = :id
            AND club_members.goat_id = goats.goat_id";
    /** @var PDO $dbConnection */
    $stmnt = $dbConnection->prepare($sql);
    $isQueryWorked = $stmnt->execute(['id' => $clubId]);
    if (!$isQueryWorked) {
        return null;
    }
    $goats = $stmnt->fetchAll();
    return $goats;
}


/**
 * Get the number of members in a club.
 * @param int $clubId The club's id.
 * @return int|null Count, or null if something broke.
 */
function getCountMembersInClub($clubId) {
    global $dbConnection;
    $sql = "SELECT COUNT(*) AS member_count 
        FROM club_members
        WHERE club_id = :id;";
    /** @var PDO $dbConnection */
    $stmnt = $dbConnection->prepare($sql);
    $isQueryWorked = $stmnt->execute(['id' => $clubId]);
    if (!$isQueryWorked) {
        return null;
    }
    $row = $stmnt->fetch();
    $memberCount = $row['member_count'];
    return $memberCount;
}

/**
 * Make HTML to render a club.
 * @param int $comedianId The comedian's id.
 * @return string|null HTML to show the club, or null if something broke.
 */
function renderComedianDeets($comedianId) {
    $comedianEntity = getComedianWithId($comedianId);
    if ( is_null($comedianEntity) ) {
        return null;
    }
    $result = "<p>Name: {$comedianEntity['name']}</p>\n";
    $result .= "<p>Comments: {$comedianEntity['comments']}</p>\n";
    // Show the members.
    $goats = getGoatsWithFaveComedian($comedianId);
    if (is_null($goats)) {
        $result .= '<p><strong>Something went wrong looking up goat who like this comedian.</strong></p>';
    }
    else {
        if (count($goats) == 0) {
            $result .= "<p><strong>This comedian is no goats' fave.</strong></p>";
        }
        else {
            $result .= "<h3>Goats with this comedian as fave</h3>\n";
            $result .= "<ul>\n";
            foreach ($goats as $goat) {
                $viewUrl = "goat-view.php?goat_id={$goat['goat_id']}";
                $viewLink = "<a href='$viewUrl'>{$goat['name']}</a>";
                $result .= "<li>$viewLink</li>\n";
            }
            $result .= "</ul>\n";
        }
    }
    return $result;
}


/**
 * Get the goats with a given comedian as their fave.
 * @param int $comedianId The comedian's id.
 * @return array|null Goats, or null if something broke.
 */
function getGoatsWithFaveComedian($comedianId) {
    global $dbConnection;
    $sql = "SELECT * 
        FROM goats
        WHERE fave_comedian = :id
        ORDER BY name;";
    $stmnt = $dbConnection->prepare($sql);
    $isQueryWorked = $stmnt->execute(['id' => $comedianId]);
    if (!$isQueryWorked) {
        return null;
    }
    $comedians = $stmnt->fetchAll();
    return $comedians;
}
/**
 * Get the number of goats with a comedian as fave.
 * @param int $comedianId The club's id.
 * @return int|null Count, or null if something broke.
 */
function getCountGoatsWithFaveComedian($comedianId) {
    global $dbConnection;
    $sql = "SELECT COUNT(*) AS goat_count 
        FROM goats
        WHERE fave_comedian = :id;";
    /** @var PDO $dbConnection */
    $stmnt = $dbConnection->prepare($sql);
    $isQueryWorked = $stmnt->execute(['id' => $comedianId]);
    if (!$isQueryWorked) {
        return null;
    }
    $row = $stmnt->fetch();
    $memberCount = $row['goat_count'];
    return $memberCount;
}

/**
 * Check a comedian id.
 * @param mixed $id Id.
 * @return string Error message, MT if OK.
 */
function checkComedianId($id) {
    global $dbConnection;
    $errorMessage = '';
    if (is_null($id)) {
        $errorMessage = 'Sorry, comedian id is missing.';
    }
    if ($errorMessage == '') {
        if (! is_numeric($id)) {
            $errorMessage = 'Sorry, comedian id must be a number.';
        }
    }
    if ($errorMessage == '') {
        if ($id < 1) {
            $errorMessage = 'Sorry, comedian id must be one or more.';
        }
    }
    if ($errorMessage == '') {
        // Prepare SQL.
        $sql = "
            SELECT *
            FROM comedians 
            WHERE comedian_id = :id;
         ";
        $stmt = $dbConnection->prepare($sql);
        // Run it.
        $isWorked = $stmt->execute(['id' => $id]);
        if (!$isWorked) {
            $errorMessage = "Sorry, a problem connecting to the database.";
        }
    }
    if ($errorMessage == '') {
        // Anything returned?
        if ($stmt->rowCount() == 0) {
            $errorMessage = "Sorry, no comedian with an id of $id.";
        }
    }
    // Return results.
    return $errorMessage;
}

/**
 * Check a club id.
 * @param mixed $id Id.
 * @return string Error message, MT if OK.
 */
function checkClubId($id) {
    global $dbConnection;
    $errorMessage = '';
    if (is_null($id)) {
        $errorMessage = 'Sorry, club id is missing.';
    }
    if ($errorMessage == '') {
        if (! is_numeric($id)) {
            $errorMessage = 'Sorry, club id must be a number.';
        }
    }
    if ($errorMessage == '') {
        if ($id < 1) {
            $errorMessage = 'Sorry, club id must be one or more.';
        }
    }
    if ($errorMessage == '') {
        // Prepare SQL.
        $sql = "
            SELECT *
            FROM clubs 
            WHERE club_id = :id;
         ";
        $stmt = $dbConnection->prepare($sql);
        // Run it.
        $isWorked = $stmt->execute(['id' => $id]);
        if (!$isWorked) {
            $errorMessage = "Sorry, a problem connecting to the database.";
        }
    }
    if ($errorMessage == '') {
        // Anything returned?
        if ($stmt->rowCount() == 0) {
            $errorMessage = "Sorry, no club with an id of $id.";
        }
    }
    // Return results.
    return $errorMessage;
}

/**
 * Check a goat id.
 * @param mixed $id Id.
 * @return string Error message, MT if OK.
 */
function checkGoatId($id) {
    global $dbConnection;
    $errorMessage = '';
    if (is_null($id)) {
        $errorMessage = 'Sorry, goat id is missing.';
    }
    if ($errorMessage == '') {
        if (! is_numeric($id)) {
            $errorMessage = 'Sorry, goat id must be a number.';
        }
    }
    if ($errorMessage == '') {
        if ($id < 1) {
            $errorMessage = 'Sorry, goat id must be one or more.';
        }
    }
    if ($errorMessage == '') {
        // Prepare SQL.
        $sql = "
            SELECT *
            FROM goats 
            WHERE goat_id = :id;
         ";
        $stmt = $dbConnection->prepare($sql);
        // Run it.
        $isWorked = $stmt->execute(['id' => $id]);
        if (!$isWorked) {
            $errorMessage = "Sorry, a problem connecting to the database.";
        }
    }
    if ($errorMessage == '') {
        // Anything returned?
        if ($stmt->rowCount() == 0) {
            $errorMessage = "Sorry, no goat with an id of $id.";
        }
    }
    // Return results.
    return $errorMessage;
}
